@@ -4,22 +4,24 @@ a bitarray.  Bitarrays are sequence types and behave very much like lists.
 
 Please find a description of this package at:
 
-    http://pypi.python.org/pypi/bitarray/
+    https://github.com/ilanschnell/bitarray
 
 Author: Ilan Schnell
 """
-from bitarray._bitarray import _bitarray, bitdiff, bits2bytes, _sysinfo
+from bitarray._bitarray import (_bitarray, bitdiff, bits2bytes, _sysinfo,
+                                get_default_endian, _set_default_endian,
+                                __version__)
 
-__version__ = '1.2.0'
+
+__all__ = ['bitarray', 'frozenbitarray', '__version__']
 
 
 class bitarray(_bitarray):
-    """bitarray(initial=0, /, endian='big')
+    """bitarray(initializer=0, /, endian='big') -> bitarray
 
 Return a new bitarray object whose items are bits initialized from
 the optional initial object, and endianness.
-If no initial object is provided, an empty bitarray (length zero) is created.
-The initial object may be of the following types:
+The initializer may be of the following types:
 
 `int`: Create a bitarray of given integer length.  The initial values are
 arbitrary.  If you want all values to be set, use the .setall() method.
@@ -40,25 +42,18 @@ Note that setting the bit endianness only has an effect when accessing the
 machine representation of the bitarray, i.e. when using the methods: tofile,
 fromfile, tobytes, frombytes."""
 
-    def fromstring(self, string):
-        """fromstring(str)
+    def tostring(self, string):
+        "NO DOC"
+        raise NotImplementedError("""
+The .tostring() / .fromstring() methods been deprecated since bitarray 0.4.0,
+and have been removed in bitarray version 1.3.0.
+Please use .tobytes() / .frombytes() instead.""")
 
-Append from a string, interpreting the string as machine values.
-Deprecated since version 0.4.0, use `.frombytes()` instead."""
-        return self.frombytes(string.encode())
-
-    def tostring(self):
-        """tostring() -> str
-
-Return the string representing (machine values) of the bitarray.
-When the length of the bitarray is not a multiple of 8, the few remaining
-bits (1..7) are set to 0.
-Deprecated since version 0.4.0, use `.tobytes()` instead."""
-        return self.tobytes().decode()
+    fromstring = tostring
 
 
 class frozenbitarray(_bitarray):
-    """frozenbitarray(initial=0, /, endian='big')
+    """frozenbitarray(initializer=0, /, endian='big') -> frozenbitarray
 
 Return a frozenbitarray object, which is initialized the same way a bitarray
 object is initialized.  A frozenbitarray is immutable and hashable.
@@ -76,7 +71,7 @@ a dictionary key.
     def __delitem__(self, *args, **kwargs):
         raise TypeError("'frozenbitarray' is immutable")
 
-    append = bytereverse = extend = encode = fill = __delitem__
+    append = bytereverse = clear = extend = encode = fill = __delitem__
     frombytes = fromfile = insert = invert = pack = pop = __delitem__
     remove = reverse = setall = sort = __setitem__ = __delitem__
     __iand__ = __iadd__ = __imul__ = __ior__ = __ixor__ = __delitem__
