@@ -7,6 +7,7 @@
 
    Author: Ilan Schnell
 */
+#define BITARRAY_VERSION  "1.4.0"
 
 #define PY_SSIZE_T_CLEAN
 #include "Python.h"
@@ -33,14 +34,13 @@
 #endif /* !STDC_HEADERS */
 
 
+/* instead of Py_ssize_t, we use this type indices, as Py_ssize_t is
+   only 4 bytes on 32bit machines, but bitarray indices can exceed this */
 typedef long long int idx_t;
 
-/* throughout:  0 = little endian   1 = big endian */
-static int default_endian = 1;
-
 /* Unlike the normal convention, ob_size is the byte count, not the number
    of elements.  The reason for doing this is that we can use our own
-   special idx_t for the number of bits (which can exceed 2^32 on a 32 bit
+   special idx_t for the number of bits, which may exceed 2^32 on a 32 bit
    machine.  */
 typedef struct {
     PyObject_VAR_HEAD
@@ -54,7 +54,12 @@ typedef struct {
 
 static PyTypeObject Bitarraytype;
 
-#define ENDIAN_STR(ba)  (((ba)->endian) ? "big" : "little")
+/* --- bit endianness --- */
+#define ENDIAN_LITTLE  0
+#define ENDIAN_BIG     1
+#define ENDIAN_INT(i)  ((i) == ENDIAN_LITTLE ? "little" : "big")
+#define ENDIAN_OBJ(o)  ENDIAN_INT(((bitarrayobject *) o)->endian)
+static int default_endian = ENDIAN_BIG;
 
 #define bitarray_Check(obj)  PyObject_TypeCheck((obj), &Bitarraytype)
 
@@ -63,7 +68,8 @@ static PyTypeObject Bitarraytype;
 /* number of bytes necessary to store given bits */
 #define BYTES(bits)  (((bits) == 0) ? 0 : (((bits) - 1) / 8 + 1))
 
-#define BITMASK(endian, i)  (((char) 1) << ((endian) ? (7 - (i)%8) : (i)%8))
+#define BITMASK(endian, i)  \
+    (((char) 1) << ((endian) == ENDIAN_LITTLE ? ((i) % 8) : (7 - (i) % 8)))
 
 /* ------------ low level access to bits in bitarrayobject ------------- */
 
@@ -98,10 +104,9 @@ check_overflow(idx_t nbits)
     if (sizeof(void *) == 4) {  /* 32bit system */
         const idx_t max_bits = ((idx_t) 1) << 34;  /* 2^34 = 16 Gbits*/
         if (nbits > max_bits) {
-            char buff[256];
-            sprintf(buff, "cannot create bitarray of size %lld, "
-                          "max size is %lld", nbits, max_bits);
-            PyErr_SetString(PyExc_OverflowError, buff);
+            PyErr_Format(PyExc_OverflowError,
+                         "cannot create bitarray of size %lld, "
+                         "max size is %lld", nbits, max_bits);
             return -1;
         }
     }
@@ -111,17 +116,35 @@ check_overflow(idx_t nbits)
 static int
 resize(bitarrayobject *self, idx_t nbits)
 {
+    const Py_ssize_t allocated = self->allocated, size = Py_SIZE(self);
     Py_ssize_t newsize;
     size_t new_allocated;
-    Py_ssize_t allocated = self->allocated;
+
+    assert(allocated >= size && size == BYTES(self->nbits));
+    /* ob_item == NULL implies ob_size == allocated == 0 */
+    assert(self->ob_item != NULL || (size == 0 && allocated == 0));
+    /* allocated ==0 implies size == 0 */
+    assert(allocated != 0 || size == 0);
 
     if (check_overflow(nbits) < 0)
         return -1;
     newsize = (Py_ssize_t) BYTES(nbits);
 
-    /* Bypass realloc() when a previous overallocation is large enough
-       to accommodate the newsize.  If the newsize falls lower than half
-       the allocated size, then proceed with the realloc() to shrink.
+    if (newsize == size) {
+        /* the memory size hasn't changed - bypass almost everything */
+        self->nbits = nbits;
+        return 0;
+    }
+
+    if (self->ob_exports > 0) {
+        PyErr_SetString(PyExc_BufferError,
+                        "cannot resize bitarray that is exporting buffers");
+        return -1;
+    }
+
+    /* Bypass reallocation when a allocation is large enough to accommodate
+       the newsize.  If the newsize falls lower than half the allocated size,
+       then proceed with the reallocation to shrink the bitarray.
     */
     if (allocated >= newsize && newsize >= (allocated >> 1)) {
         assert(self->ob_item != NULL || newsize == 0);
@@ -130,21 +153,36 @@ resize(bitarrayobject *self, idx_t nbits)
         return 0;
     }
 
+    if (newsize == 0) {
+        PyMem_FREE(self->ob_item);
+        self->ob_item = NULL;
+        Py_SIZE(self) = 0;
+        self->allocated = 0;
+        self->nbits = 0;
+        return 0;
+    }
+
     new_allocated = (size_t) newsize;
-    if (newsize < Py_SIZE(self) + 65536)
-        /* Over-allocate unless the size increase is very large.
-           This over-allocates proportional to the bitarray size, making
+    if (size == 0 && newsize <= 4)
+        /* When resizing an empty bitarray, we want at least 4 bytes. */
+        new_allocated = 4;
+
+    /* Over-allocate unless (one of the following):
+         - The (previous) size is zero, as we often
+           extend an empty array on creation.
+         - The size increase is very large.
+         - The size is decreasing. */
+    else if (!(size == 0 || newsize > size + 65536 || newsize < size))
+        /* This over-allocates proportional to the bitarray size, making
            room for additional growth.
            The growth pattern is:  0, 4, 8, 16, 25, 34, 44, 54, 65, 77, ...
-           Note, the pattern starts out the same as for lists but then
+           The pattern starts out the same as for lists but then
            grows at a smaller rate so that larger bitarrays only overallocate
            by about 1/16th -- this is done because bitarrays are assumed
-           to be memory critical.
-        */
+           to be memory critical. */
         new_allocated += (newsize >> 4) + (newsize < 8 ? 3 : 7);
 
-    if (newsize == 0)
-        new_allocated = 0;
+    assert(new_allocated >= (size_t) newsize);
     self->ob_item = PyMem_Realloc(self->ob_item, new_allocated);
     if (self->ob_item == NULL) {
         PyErr_NoMemory();
@@ -172,8 +210,6 @@ newbitarrayobject(PyTypeObject *type, idx_t nbits, int endian)
 
     nbytes = (Py_ssize_t) BYTES(nbits);
     Py_SIZE(obj) = nbytes;
-    obj->nbits = nbits;
-    obj->endian = endian;
     if (nbytes == 0) {
         obj->ob_item = NULL;
     }
@@ -186,6 +222,9 @@ newbitarrayobject(PyTypeObject *type, idx_t nbits, int endian)
         }
     }
     obj->allocated = nbytes;
+    obj->nbits = nbits;
+    obj->endian = endian;
+    obj->ob_exports = 0;
     obj->weakreflist = NULL;
     return (PyObject *) obj;
 }
@@ -285,15 +324,11 @@ setunused(bitarrayobject *self)
 {
     const idx_t n = BITS(Py_SIZE(self));
     idx_t i;
-    int res = 0;
 
-    for (i = self->nbits; i < n; i++) {
+    for (i = self->nbits; i < n; i++)
         setbit(self, i, 0);
-        res++;
-    }
-    assert(res < 8);
-    assert(n == self->nbits + res);
-    return res;
+    assert(n - self->nbits < 8);
+    return n - self->nbits;
 }
 
 /* repeat self n times */
@@ -328,11 +363,11 @@ static int
 bitwise(bitarrayobject *self, PyObject *arg, enum op_type oper)
 {
     bitarrayobject *other;
-    Py_ssize_t i;
+    Py_ssize_t n = Py_SIZE(self), i;
 
     if (!bitarray_Check(arg)) {
         PyErr_SetString(PyExc_TypeError,
-                        "bitarray object expected for bitwise operation");
+                        "bitarray expected for bitwise operation");
         return -1;
     }
     other = (bitarrayobject *) arg;
@@ -345,15 +380,15 @@ bitwise(bitarrayobject *self, PyObject *arg, enum op_type oper)
     setunused(other);
     switch (oper) {
     case OP_and:
-        for (i = 0; i < Py_SIZE(self); i++)
+        for (i = 0; i < n; i++)
             self->ob_item[i] &= other->ob_item[i];
         break;
     case OP_or:
-        for (i = 0; i < Py_SIZE(self); i++)
+        for (i = 0; i < n; i++)
             self->ob_item[i] |= other->ob_item[i];
         break;
     case OP_xor:
-        for (i = 0; i < Py_SIZE(self); i++)
+        for (i = 0; i < n; i++)
             self->ob_item[i] ^= other->ob_item[i];
         break;
     default:  /* should never happen */
@@ -406,12 +441,12 @@ bytereverse(bitarrayobject *self)
     static char trans[256];
     static int setup = 0;
     Py_ssize_t i;
-    unsigned char c;
 
     if (!setup) {
         /* setup translation table, which maps each byte to it's reversed:
            trans = {0, 128, 64, 192, 32, 160, ..., 255} */
         int j, k;
+
         for (k = 0; k < 256; k++) {
             trans[k] = 0x00;
             for (j = 0; j < 8; j++)
@@ -422,37 +457,26 @@ bytereverse(bitarrayobject *self)
     }
 
     setunused(self);
-    for (i = 0; i < Py_SIZE(self); i++) {
-        c = self->ob_item[i];
-        self->ob_item[i] = trans[c];
-    }
+    for (i = 0; i < Py_SIZE(self); i++)
+        self->ob_item[i] = trans[(unsigned char) self->ob_item[i]];
 }
 
 static unsigned char bitcount_lookup[256] = {
-    0, 1, 1, 2, 1, 2, 2, 3, 1, 2, 2, 3, 2, 3, 3, 4,
-    1, 2, 2, 3, 2, 3, 3, 4, 2, 3, 3, 4, 3, 4, 4, 5,
-    1, 2, 2, 3, 2, 3, 3, 4, 2, 3, 3, 4, 3, 4, 4, 5,
-    2, 3, 3, 4, 3, 4, 4, 5, 3, 4, 4, 5, 4, 5, 5, 6,
-    1, 2, 2, 3, 2, 3, 3, 4, 2, 3, 3, 4, 3, 4, 4, 5,
-    2, 3, 3, 4, 3, 4, 4, 5, 3, 4, 4, 5, 4, 5, 5, 6,
-    2, 3, 3, 4, 3, 4, 4, 5, 3, 4, 4, 5, 4, 5, 5, 6,
-    3, 4, 4, 5, 4, 5, 5, 6, 4, 5, 5, 6, 5, 6, 6, 7,
-    1, 2, 2, 3, 2, 3, 3, 4, 2, 3, 3, 4, 3, 4, 4, 5,
-    2, 3, 3, 4, 3, 4, 4, 5, 3, 4, 4, 5, 4, 5, 5, 6,
-    2, 3, 3, 4, 3, 4, 4, 5, 3, 4, 4, 5, 4, 5, 5, 6,
-    3, 4, 4, 5, 4, 5, 5, 6, 4, 5, 5, 6, 5, 6, 6, 7,
-    2, 3, 3, 4, 3, 4, 4, 5, 3, 4, 4, 5, 4, 5, 5, 6,
-    3, 4, 4, 5, 4, 5, 5, 6, 4, 5, 5, 6, 5, 6, 6, 7,
-    3, 4, 4, 5, 4, 5, 5, 6, 4, 5, 5, 6, 5, 6, 6, 7,
-    4, 5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8,
+    0,1,1,2,1,2,2,3,1,2,2,3,2,3,3,4,1,2,2,3,2,3,3,4,2,3,3,4,3,4,4,5,
+    1,2,2,3,2,3,3,4,2,3,3,4,3,4,4,5,2,3,3,4,3,4,4,5,3,4,4,5,4,5,5,6,
+    1,2,2,3,2,3,3,4,2,3,3,4,3,4,4,5,2,3,3,4,3,4,4,5,3,4,4,5,4,5,5,6,
+    2,3,3,4,3,4,4,5,3,4,4,5,4,5,5,6,3,4,4,5,4,5,5,6,4,5,5,6,5,6,6,7,
+    1,2,2,3,2,3,3,4,2,3,3,4,3,4,4,5,2,3,3,4,3,4,4,5,3,4,4,5,4,5,5,6,
+    2,3,3,4,3,4,4,5,3,4,4,5,4,5,5,6,3,4,4,5,4,5,5,6,4,5,5,6,5,6,6,7,
+    2,3,3,4,3,4,4,5,3,4,4,5,4,5,5,6,3,4,4,5,4,5,5,6,4,5,5,6,5,6,6,7,
+    3,4,4,5,4,5,5,6,4,5,5,6,5,6,6,7,4,5,5,6,5,6,6,7,5,6,6,7,6,7,7,8,
 };
 
-/* returns number of 1 bits */
+/* Return number of 1 bits.  This function never fails. */
 static idx_t
 count(bitarrayobject *self, int vi, idx_t start, idx_t stop)
 {
     idx_t i, res = 0;
-    unsigned char c;
 
     assert(0 <= start && start <= self->nbits);
     assert(0 <= stop && stop <= self->nbits);
@@ -469,10 +493,8 @@ count(bitarrayobject *self, int vi, idx_t start, idx_t stop)
 
         for (i = start; i < BITS(byte_start); i++)
             res += GETBIT(self, i);
-        for (j = byte_start; j < byte_stop; j++) {
-            c = self->ob_item[j];
-            res += bitcount_lookup[c];
-        }
+        for (j = byte_start; j < byte_stop; j++)
+            res += bitcount_lookup[(unsigned char) self->ob_item[j]];
         for (i = BITS(byte_stop); i < stop; i++)
             res += GETBIT(self, i);
     }
@@ -483,7 +505,8 @@ count(bitarrayobject *self, int vi, idx_t start, idx_t stop)
     return vi ? res : stop - start - res;
 }
 
-/* return index of first occurrence of vi, -1 when x is not in found. */
+/* Return index of first occurrence of vi, -1 when x is not in found.
+   This function never fails. */
 static idx_t
 findfirst(bitarrayobject *self, int vi, idx_t start, idx_t stop)
 {
@@ -544,7 +567,7 @@ search(bitarrayobject *self, bitarrayobject *xa, idx_t p)
 static int
 set_item(bitarrayobject *self, idx_t i, PyObject *v)
 {
-    long vi;
+    int vi;
 
     assert(0 <= i && i < self->nbits);
     vi = PyObject_IsTrue(v);
@@ -579,7 +602,7 @@ unpack(bitarrayobject *self, char zero, char one)
         return NULL;
     }
     for (i = 0; i < self->nbits; i++) {
-        *(str + i) = GETBIT(self, i) ? one : zero;
+        str[i] = GETBIT(self, i) ? one : zero;
     }
     result = PyBytes_FromStringAndSize(str, (Py_ssize_t) self->nbits);
     PyMem_Free((void *) str);
@@ -589,20 +612,18 @@ unpack(bitarrayobject *self, char zero, char one)
 static int
 extend_bitarray(bitarrayobject *self, bitarrayobject *other)
 {
-    idx_t self_bits, other_bits;
+    /* We have to store the sizes before we resize, and since
+       other may be self, we also need to store other->nbits. */
+    idx_t self_nbits = self->nbits;
+    idx_t other_nbits = other->nbits;
 
-    if (other->nbits == 0)
+    if (other_nbits == 0)
         return 0;
 
-    /* We have to store the sizes before we resize.
-       Since other may be self, we also need to store other->nbits. */
-    self_bits = self->nbits;
-    other_bits = other->nbits;
-
-    if (resize(self, self_bits + other_bits) < 0)
+    if (resize(self, self_nbits + other_nbits) < 0)
         return -1;
 
-    copy_n(self, self_bits, other, 0, other_bits);
+    copy_n(self, self_nbits, other, 0, other_nbits);
     return 0;
 }
 
@@ -683,22 +704,22 @@ enum conv_t {
 static int
 extend_bytes(bitarrayobject *self, PyObject *bytes, enum conv_t conv)
 {
-    Py_ssize_t strlen, i;
-    char c, *str;
+    Py_ssize_t nbytes, i;
+    char c, *data;
     int vi = 0;
 
     assert(PyBytes_Check(bytes));
-    strlen = PyBytes_Size(bytes);
-    if (strlen == 0)
+    nbytes = PyBytes_Size(bytes);
+    if (nbytes == 0)
         return 0;
 
-    if (resize(self, self->nbits + strlen) < 0)
+    if (resize(self, self->nbits + nbytes) < 0)
         return -1;
 
-    str = PyBytes_AsString(bytes);
+    data = PyBytes_AsString(bytes);
 
-    for (i = 0; i < strlen; i++) {
-        c = *(str + i);
+    for (i = 0; i < nbytes; i++) {
+        c = data[i];
         /* depending on conv, map c to bit */
         switch (conv) {
         case BYTES_01:
@@ -717,7 +738,7 @@ extend_bytes(bitarrayobject *self, PyObject *bytes, enum conv_t conv)
         default:  /* should never happen */
             return -1;
         }
-        setbit(self, self->nbits - strlen + i, vi);
+        setbit(self, self->nbits - nbytes + i, vi);
     }
     return 0;
 }
@@ -725,19 +746,19 @@ extend_bytes(bitarrayobject *self, PyObject *bytes, enum conv_t conv)
 static int
 extend_rawbytes(bitarrayobject *self, PyObject *bytes)
 {
-    Py_ssize_t strlen;
-    char *str;
+    Py_ssize_t nbytes;
+    char *data;
 
     assert(PyBytes_Check(bytes) && self->nbits % 8 == 0);
-    strlen = PyBytes_Size(bytes);
-    if (strlen == 0)
+    nbytes = PyBytes_Size(bytes);
+    if (nbytes == 0)
         return 0;
 
-    if (resize(self, self->nbits + BITS(strlen)) < 0)
+    if (resize(self, self->nbits + BITS(nbytes)) < 0)
         return -1;
 
-    str = PyBytes_AsString(bytes);
-    memcpy(self->ob_item + (Py_SIZE(self) - strlen), str, strlen);
+    data = PyBytes_AsString(bytes);
+    memcpy(self->ob_item + (Py_SIZE(self) - nbytes), data, nbytes);
     return 0;
 }
 
@@ -773,13 +794,14 @@ extend_dispatch(bitarrayobject *self, PyObject *obj)
 
     /* finally, try to get the iterator of the object */
     iter = PyObject_GetIter(obj);
-    if (iter == NULL) {
-        PyErr_SetString(PyExc_TypeError, "could not extend bitarray");
-        return -1;
+    if (iter) {
+        ret = extend_iter(self, iter);
+        Py_DECREF(iter);
+        return ret;
     }
-    ret = extend_iter(self, iter);
-    Py_DECREF(iter);
-    return ret;
+    PyErr_Format(PyExc_TypeError,
+                 "'%s' object is not iterable", Py_TYPE(obj)->tp_name);
+    return -1;
 }
 
 /* --------- helper functions not involving bitarrayobjects ------------ */
@@ -787,12 +809,10 @@ extend_dispatch(bitarrayobject *self, PyObject *obj)
 #ifdef IS_PY3K
 #define IS_INDEX(x)  (PyLong_Check(x) || PyIndex_Check(x))
 #define IS_INT_OR_BOOL(x)  (PyBool_Check(x) || PyLong_Check(x))
-#define Py_CSTRING  PyUnicode_FromString
 #else  /* Py 2 */
 #define IS_INDEX(x)  (PyInt_Check(x) || PyLong_Check(x) || PyIndex_Check(x))
 #define IS_INT_OR_BOOL(x)  (PyBool_Check(x) || PyInt_Check(x) || \
                                                PyLong_Check(x))
-#define Py_CSTRING  PyString_FromString
 #endif
 
 /* given an PyLong (which must be 0 or 1), or a PyBool, return 0 or 1,
@@ -981,7 +1001,7 @@ bitarray_count(bitarrayobject *self, PyObject *args)
 {
     PyObject *x = Py_True;
     idx_t start = 0, stop = self->nbits;
-    long vi;
+    int vi;
 
     if (!PyArg_ParseTuple(args, "|OLL:count", &x, &start, &stop))
         return NULL;
@@ -1007,7 +1027,7 @@ bitarray_index(bitarrayobject *self, PyObject *args)
 {
     PyObject *x;
     idx_t i, start = 0, stop = self->nbits;
-    long vi;
+    int vi;
 
     if (!PyArg_ParseTuple(args, "O|LL:index", &x, &start, &stop))
         return NULL;
@@ -1021,7 +1041,7 @@ bitarray_index(bitarrayobject *self, PyObject *args)
 
     i = findfirst(self, vi, start, stop);
     if (i < 0) {
-        PyErr_SetString(PyExc_ValueError, "index(x): x not in bitarray");
+        PyErr_Format(PyExc_ValueError, "%d is not in bitarray", vi);
         return NULL;
     }
     return PyLong_FromLongLong(i);
@@ -1144,7 +1164,7 @@ bitarray_buffer_info(bitarrayobject *self)
     res = Py_BuildValue("OLsiL",
                         ptr,
                         (idx_t) Py_SIZE(self),
-                        ENDIAN_STR(self),
+                        ENDIAN_OBJ(self),
                         (int) (BITS(Py_SIZE(self)) - self->nbits),
                         (idx_t) self->allocated);
     Py_DECREF(ptr);
@@ -1155,16 +1175,15 @@ PyDoc_STRVAR(buffer_info_doc,
 "buffer_info() -> tuple\n\
 \n\
 Return a tuple (address, size, endianness, unused, allocated) giving the\n\
-current memory address, the size (in bytes) used to hold the bitarray's\n\
-contents, the bit endianness as a string, the number of unused bits\n\
-(e.g. a bitarray of length 11 will have a buffer size of 2 bytes and\n\
-5 unused bits), and the size (in bytes) of the allocated memory.");
+memory address of the bitarray's data, the size (in bytes) used to hold the\n\
+bitarray's contents, the bit endianness as a string, the number of unused\n\
+bits in the last bytes, and the size (in bytes) of the allocated memory.");
 
 
 static PyObject *
 bitarray_endian(bitarrayobject *self)
 {
-    return Py_CSTRING(ENDIAN_STR(self));
+    return Py_BuildValue("s", ENDIAN_OBJ(self));
 }
 
 PyDoc_STRVAR(endian_doc,
@@ -1178,7 +1197,6 @@ bitarray_append(bitarrayobject *self, PyObject *v)
 {
     if (append_item(self, v) < 0)
         return NULL;
-
     Py_RETURN_NONE;
 }
 
@@ -1191,10 +1209,7 @@ Append the value `bool(item)` to the end of the bitarray.");
 static PyObject *
 bitarray_all(bitarrayobject *self)
 {
-    if (findfirst(self, 0, 0, self->nbits) >= 0)
-        Py_RETURN_FALSE;
-    else
-        Py_RETURN_TRUE;
+    return PyBool_FromLong(findfirst(self, 0, 0, self->nbits) == -1);
 }
 
 PyDoc_STRVAR(all_doc,
@@ -1206,10 +1221,7 @@ Returns True when all bits in the array are True.");
 static PyObject *
 bitarray_any(bitarrayobject *self)
 {
-    if (findfirst(self, 1, 0, self->nbits) >= 0)
-        Py_RETURN_TRUE;
-    else
-        Py_RETURN_FALSE;
+    return PyBool_FromLong(findfirst(self, 1, 0, self->nbits) >= 0);
 }
 
 PyDoc_STRVAR(any_doc,
@@ -1222,7 +1234,8 @@ static PyObject *
 bitarray_reduce(bitarrayobject *self)
 {
     PyObject *dict, *repr = NULL, *result = NULL;
-    char *str;
+    Py_ssize_t nbytes = Py_SIZE(self);
+    char *data;
 
     dict = PyObject_GetAttrString((PyObject *) self, "__dict__");
     if (dict == NULL) {
@@ -1232,19 +1245,19 @@ bitarray_reduce(bitarrayobject *self)
     }
     /* the first byte indicates the number of unused bits at the end, and
        the rest of the bytes consist of the raw binary data */
-    str = (char *) PyMem_Malloc(Py_SIZE(self) + 1);
-    if (str == NULL) {
+    data = (char *) PyMem_Malloc(nbytes + 1);
+    if (data == NULL) {
         PyErr_NoMemory();
         goto error;
     }
-    str[0] = (char) setunused(self);
-    memcpy(str + 1, self->ob_item, Py_SIZE(self));
-    repr = PyBytes_FromStringAndSize(str, Py_SIZE(self) + 1);
+    data[0] = (char) setunused(self);
+    memcpy(data + 1, self->ob_item, nbytes);
+    repr = PyBytes_FromStringAndSize(data, nbytes + 1);
     if (repr == NULL)
         goto error;
-    PyMem_Free((void *) str);
+    PyMem_Free((void *) data);
     result = Py_BuildValue("O(Os)O", Py_TYPE(self),
-                           repr, ENDIAN_STR(self), dict);
+                           repr, ENDIAN_OBJ(self), dict);
 error:
     Py_DECREF(dict);
     Py_XDECREF(repr);
@@ -1257,31 +1270,30 @@ PyDoc_STRVAR(reduce_doc, "state information for pickling");
 static PyObject *
 bitarray_reverse(bitarrayobject *self)
 {
-    PyObject *t;    /* temp bitarray to store lower half of self */
-    idx_t i, m;
+    const idx_t m = self->nbits - 1;   /* index of max item of self */
+    bitarrayobject *t;    /* temp bitarray to store lower half of self */
+    idx_t i;
 
-    if (self->nbits < 2)
+    if (self->nbits < 2)        /* nothing needs to be done */
         Py_RETURN_NONE;
 
-    t = newbitarrayobject(Py_TYPE(self), self->nbits / 2, self->endian);
+    t = (bitarrayobject *) newbitarrayobject(Py_TYPE(self),
+                                             self->nbits / 2, self->endian);
     if (t == NULL)
         return NULL;
 
-#define tt  ((bitarrayobject *) t)
     /* copy lower half of array into temporary array */
-    memcpy(tt->ob_item, self->ob_item, Py_SIZE(tt));
-
-    m = self->nbits - 1;
+    memcpy(t->ob_item, self->ob_item, Py_SIZE(t));
 
     /* reverse the upper half onto the lower half. */
-    for (i = 0; i < tt->nbits; i++)
+    for (i = 0; i < t->nbits; i++)
         setbit(self, i, GETBIT(self, m - i));
 
     /* revert the stored away lower half onto the upper half. */
-    for (i = 0; i < tt->nbits; i++)
-        setbit(self, m - i, GETBIT(tt, i));
-#undef tt
-    Py_DECREF(t);
+    for (i = 0; i < t->nbits; i++)
+        setbit(self, m - i, GETBIT(t, i));
+
+    bitarray_dealloc(t);
     Py_RETURN_NONE;
 }
 
@@ -1298,11 +1310,7 @@ bitarray_fill(bitarrayobject *self)
 
     p = setunused(self);
     self->nbits += p;
-#ifdef IS_PY3K
     return PyLong_FromLong(p);
-#else
-    return PyInt_FromLong(p);
-#endif
 }
 
 PyDoc_STRVAR(fill_doc,
@@ -1344,7 +1352,7 @@ bitarray; it does not change the endianness of the bitarray object.");
 static PyObject *
 bitarray_setall(bitarrayobject *self, PyObject *v)
 {
-    long vi;
+    int vi;
 
     vi = PyObject_IsTrue(v);
     if (vi < 0)
@@ -1846,7 +1854,7 @@ static PyObject *
 bitarray_remove(bitarrayobject *self, PyObject *v)
 {
     idx_t i;
-    long vi;
+    int vi;
 
     vi = PyObject_IsTrue(v);
     if (vi < 0)
@@ -1854,7 +1862,7 @@ bitarray_remove(bitarrayobject *self, PyObject *v)
 
     i = findfirst(self, vi, 0, self->nbits);
     if (i < 0) {
-        PyErr_SetString(PyExc_ValueError, "remove(x): x not in bitarray");
+        PyErr_Format(PyExc_ValueError, "%d not in bitarray", vi);
         return NULL;
     }
     if (delete_n(self, i, 1) < 0)
@@ -1869,6 +1877,20 @@ Remove the first occurrence of `bool(value)` in the bitarray.\n\
 Raises `ValueError` if item is not present.");
 
 
+static PyObject *
+bitarray_clear(bitarrayobject *self)
+{
+    if (resize(self, 0) < 0)
+        return NULL;
+    Py_RETURN_NONE;
+}
+
+PyDoc_STRVAR(clear_doc,
+"clear()\n\
+\n\
+Remove all items from the bitarray.");
+
+
 /* --------- special methods ----------- */
 
 static PyObject *
@@ -1907,8 +1929,7 @@ bitarray_getitem(bitarrayobject *self, PyObject *a)
 }
 
 /* Sets the elements, specified by slice, in self to the value(s) given by v
-   which is either a bitarray or a boolean.
-*/
+   which is either a bitarray or a boolean. */
 static int
 setslice(bitarrayobject *self, PySliceObject *slice, PyObject *v)
 {
@@ -1926,11 +1947,10 @@ setslice(bitarrayobject *self, PySliceObject *slice, PyObject *v)
             return 0;
         }
         if (step != 1) {
-            char buff[256];
-            sprintf(buff, "attempt to assign sequence of size %lld "
-                          "to extended slice of size %lld",
-                    vv->nbits, (idx_t) slicelength);
-            PyErr_SetString(PyExc_ValueError, buff);
+            PyErr_Format(PyExc_ValueError,
+                         "attempt to assign sequence of size %lld "
+                         "to extended slice of size %lld",
+                         vv->nbits, (idx_t) slicelength);
             return -1;
         }
         /* make self bigger or smaller */
@@ -2222,7 +2242,7 @@ PyDoc_STRVAR(encode_doc,
 \n\
 Given a prefix code (a dict mapping symbols to bitarrays),\n\
 iterate over the iterable object with symbols, and extend the bitarray\n\
-with the corresponding bitarray for each symbols.");
+with the corresponding bitarray for each symbol.");
 
 
 /* Binary tree definition */
@@ -2260,6 +2280,7 @@ delete_binode_tree(binode *tree)
     PyMem_Free(tree);
 }
 
+/* insert symbol (mapping to ba) into the tree */
 static int
 insert_symbol(binode *tree, bitarrayobject *ba, PyObject *symbol)
 {
@@ -2295,6 +2316,8 @@ insert_symbol(binode *tree, bitarrayobject *ba, PyObject *symbol)
     return -1;
 }
 
+/* return a binary tree from a codedict, which is created by inserting
+   all symbols mapping to bitarrays */
 static binode *
 make_tree(PyObject *codedict)
 {
@@ -2315,11 +2338,10 @@ make_tree(PyObject *codedict)
     return tree;
 }
 
-/*
-  Traverse tree using the branches corresponding to the bitarray `ba`,
-  starting at *indexp.  Return the symbol at the leaf node, or NULL
-  when the end of the bitarray has been reached, or on error (in which
-  case the appropriate PyErr_SetString is set.
+/* Traverse tree using the branches corresponding to the bitarray `ba`,
+   starting at *indexp.  Return the symbol at the leaf node, or NULL
+   when the end of the bitarray has been reached, or on error (in which
+   case the appropriate PyErr_SetString is set.
 */
 static PyObject *
 traverse_tree(binode *tree, bitarrayobject *ba, idx_t *indexp)
@@ -2349,7 +2371,7 @@ static PyObject *
 bitarray_decode(bitarrayobject *self, PyObject *codedict)
 {
     binode *tree, *nd;
-    PyObject *list;
+    PyObject *list = NULL;
     Py_ssize_t i;
     int k;
 
@@ -2358,14 +2380,13 @@ bitarray_decode(bitarrayobject *self, PyObject *codedict)
 
     tree = make_tree(codedict);
     if (tree == NULL || PyErr_Occurred())
-        return NULL;
+        goto error;
 
     nd = tree;
     list = PyList_New(0);
-    if (list == NULL) {
-        delete_binode_tree(tree);
-        return NULL;
-    }
+    if (list == NULL)
+        goto error;
+
     /* traverse tree (just like above) */
     for (i = 0; i < self->nbits; i++) {
         k = GETBIT(self, i);
@@ -2390,7 +2411,7 @@ bitarray_decode(bitarrayobject *self, PyObject *codedict)
 
 error:
     delete_binode_tree(tree);
-    Py_DECREF(list);
+    Py_XDECREF(list);
     return NULL;
 }
 
@@ -2402,7 +2423,6 @@ decode the content of the bitarray and return it as a list of symbols.");
 
 /*********************** (Bitarray) Decode Iterator *********************/
 
-
 typedef struct {
     PyObject_HEAD
     bitarrayobject *bao;        /* bitarray we're searching in */
@@ -2415,12 +2435,11 @@ static PyTypeObject DecodeIter_Type;
 #define DecodeIter_Check(op)  PyObject_TypeCheck(op, &DecodeIter_Type)
 
 
-
-/* create a new initialized bitarray search iterator object */
+/* create a new initialized bitarray decode iterator object */
 static PyObject *
 bitarray_iterdecode(bitarrayobject *self, PyObject *codedict)
 {
-    decodeiterobject *it;  /* iterator to be returned */
+    decodeiterobject *it;       /* iterator to be returned */
     binode *tree;
 
     if (check_codedict(codedict) < 0)
@@ -2645,6 +2664,8 @@ bitarray_methods[] = {
      buffer_info_doc},
     {"bytereverse",  (PyCFunction) bitarray_bytereverse, METH_NOARGS,
      bytereverse_doc},
+    {"clear",        (PyCFunction) bitarray_clear,       METH_NOARGS,
+     clear_doc},
     {"copy",         (PyCFunction) bitarray_copy,        METH_NOARGS,
      copy_doc},
     {"count",        (PyCFunction) bitarray_count,       METH_VARARGS,
@@ -2742,18 +2763,19 @@ bitarray_methods[] = {
 static int
 endian_from_string(const char* string)
 {
-    assert(default_endian == 0 || default_endian == 1);
+    assert(default_endian == ENDIAN_LITTLE || default_endian == ENDIAN_BIG);
 
     if (string == NULL)
         return default_endian;
 
     if (strcmp(string, "little") == 0)
-        return 0;
+        return ENDIAN_LITTLE;
 
     if (strcmp(string, "big") == 0)
-        return 1;
+        return ENDIAN_BIG;
 
-    PyErr_SetString(PyExc_ValueError, "endian must be 'little' or 'big'");
+    PyErr_SetString(PyExc_ValueError,
+                    "bit endianness must be either 'little' or 'big'");
     return -1;
 }
 
@@ -2781,7 +2803,7 @@ bitarray_new(PyTypeObject *type, PyObject *args, PyObject *kwds)
 
     /* int, long */
     if (IS_INDEX(initial)) {
-        idx_t nbits = 0;
+        idx_t nbits;
 
         if (getIndex(initial, &nbits) < 0)
             return NULL;
@@ -2805,44 +2827,34 @@ bitarray_new(PyTypeObject *type, PyObject *args, PyObject *kwds)
         return a;
     }
 
-    /* bytes */
+    /* bytes (for pickling) */
     if (PyBytes_Check(initial)) {
-        Py_ssize_t strlen;
-        char *str;
+        Py_ssize_t nbytes;
+        char *data;
 
-        strlen = PyBytes_Size(initial);
-        if (strlen == 0)        /* empty string */
+        nbytes = PyBytes_Size(initial);
+        if (nbytes == 0)        /* no bytes */
             return newbitarrayobject(type, 0, endian);
 
-        str = PyBytes_AsString(initial);
-        if (0 <= str[0] && str[0] < 8) {
+        data = PyBytes_AsString(initial);
+        if (0 <= data[0] && data[0] < 8) {
             /* when the first character is smaller than 8, it indicates the
                number of unused bits at the end, and rest of the bytes
-               consist of the raw binary data, this is used for pickling */
-            if (strlen == 1 && str[0] > 0) {
+               consist of the raw binary data */
+            if (nbytes == 1 && data[0] > 0) {
                 PyErr_Format(PyExc_ValueError,
-                             "did not expect 0x0%d", (int) str[0]);
+                             "did not expect 0x0%d", (int) data[0]);
                 return NULL;
             }
-            a = newbitarrayobject(type, BITS(strlen - 1) - ((idx_t) str[0]),
+            a = newbitarrayobject(type, BITS(nbytes - 1) - ((idx_t) data[0]),
                                   endian);
             if (a == NULL)
                 return NULL;
-            memcpy(((bitarrayobject *) a)->ob_item, str + 1, strlen - 1);
+            memcpy(((bitarrayobject *) a)->ob_item, data + 1, nbytes - 1);
             return a;
         }
     }
 
-#define CHECK_TYPE(type)  \
-    if (Py ## type ## _Check(initial)) {                                  \
-        PyErr_SetString(PyExc_TypeError,                                  \
-                        "cannot create bitarray from " #type " object");  \
-        return NULL;                                                      \
-    }
-CHECK_TYPE(Float)
-CHECK_TYPE(Complex)
-#undef CHECK_TYPE
-
     /* leave remaining type dispatch to the extend method */
     a = newbitarrayobject(type, 0, endian);
     if (a == NULL)
@@ -2931,7 +2943,7 @@ static PyTypeObject BitarrayIter_Type;
 #define BitarrayIter_Check(op)  PyObject_TypeCheck(op, &BitarrayIter_Type)
 
 /* create a new initialized bitarray iterator object, this object is
-   returned when calling item(a) */
+   returned when calling iter(a) */
 static PyObject *
 bitarray_iter(bitarrayobject *self)
 {
@@ -3103,14 +3115,14 @@ static PyBufferProcs bitarray_as_buffer = {
 
 #endif  /* WITH_BUFFER */
 
+/************************** Bitarray Type *******************************/
+
 PyDoc_STRVAR(bitarraytype_doc,
 "_bitarray(initializer=0, /, endian='big') -> bitarray\n\
 \n\
 Return a new bitarray object.\n\
 ");
 
-/************************** Bitarray Type *******************************/
-
 static PyTypeObject Bitarraytype = {
 #ifdef IS_PY3K
     PyVarObject_HEAD_INIT(NULL, 0)
@@ -3170,7 +3182,7 @@ static PyTypeObject Bitarraytype = {
 /*************************** Module functions **********************/
 
 static PyObject *
-bitdiff(PyObject *self, PyObject *args)
+bitdiff(PyObject *module, PyObject *args)
 {
     PyObject *a, *b;
     Py_ssize_t i;
@@ -3212,7 +3224,7 @@ Deprecated since version 1.2.0, use `bitarray.util.count_xor()` instead.");
 
 
 static PyObject *
-bits2bytes(PyObject *self, PyObject *v)
+bits2bytes(PyObject *module, PyObject *v)
 {
     idx_t n = 0;
 
@@ -3236,9 +3248,9 @@ Return the number of bytes necessary to store n bits.");
 
 
 static PyObject *
-get_default_endian(PyObject *self)
+get_default_endian(PyObject *module)
 {
-    return Py_CSTRING(default_endian ? "big" : "little");
+    return Py_BuildValue("s", ENDIAN_INT(default_endian));
 }
 
 PyDoc_STRVAR(get_default_endian_doc,
@@ -3249,7 +3261,7 @@ Under normal circumstances, the return value is `big`.");
 
 
 static PyObject *
-set_default_endian(PyObject *self, PyObject *args)
+set_default_endian(PyObject *module, PyObject *args)
 {
     char *endian_str;
     int tmp;
@@ -3294,7 +3306,8 @@ tuple(sizeof(void *),\n\
       PY_SSIZE_T_MAX)");
 
 /*
-   In retrospect, I wish I had never added any modules functions here.
+   In retrospect, I wish I had never added any modules functions (other
+   than get_default_endian() and _set_default_endian()) here.
    These, and possibly many others should be part of a separate utility
    module.  Anyway, at this point (2019) it is too late to remove them,
    so I will just leave them here, but not any new ones.
@@ -3316,10 +3329,12 @@ static PyMethodDef module_functions[] = {
 static PyModuleDef moduledef = {
     PyModuleDef_HEAD_INIT, "_bitarray", 0, -1, module_functions,
 };
+#endif
+
 PyMODINIT_FUNC
+#ifdef IS_PY3K
 PyInit__bitarray(void)
 #else
-PyMODINIT_FUNC
 init_bitarray(void)
 #endif
 {
@@ -3341,6 +3356,8 @@ init_bitarray(void)
 
     Py_INCREF((PyObject *) &Bitarraytype);
     PyModule_AddObject(m, "_bitarray", (PyObject *) &Bitarraytype);
+    PyModule_AddObject(m, "__version__",
+                       Py_BuildValue("s", BITARRAY_VERSION));
 #ifdef IS_PY3K
     return m;
 #endif
