@@ -1,5 +1,5 @@
 /*
-   Copyright (c) 2019, Ilan Schnell
+   Copyright (c) 2019 - 2020, Ilan Schnell
    bitarray is published under the PSF license.
 
    This file contains the C implementation of some useful utility functions.
@@ -35,11 +35,15 @@ typedef struct {
     PyObject *weakreflist;      /* list of weak references */
 } bitarrayobject;
 
+#define ENDIAN_LITTLE  0
+#define ENDIAN_BIG     1
+
 #define BITS(bytes)  ((idx_t) (bytes) << 3)
 
 #define BYTES(bits)  (((bits) == 0) ? 0 : (((bits) - 1) / 8 + 1))
 
-#define BITMASK(endian, i)  (((char) 1) << ((endian) ? (7 - (i)%8) : (i)%8))
+#define BITMASK(endian, i)  \
+    (((char) 1) << ((endian) == ENDIAN_LITTLE ? ((i) % 8) : (7 - (i) % 8)))
 
 /* ------------ low level access to bits in bitarrayobject ------------- */
 
@@ -67,47 +71,46 @@ setbit(bitarrayobject *self, idx_t i, int bit)
         *cp &= ~mask;
 }
 
-/* set using the Python module function _set_babt() */
-static PyObject *bitarray_basetype = NULL;
-
-/* return 1 if obj is a bitarray, 0 otherwise */
-static int
-bitarray_Check(PyObject *obj)
-{
-    if (bitarray_basetype == NULL) /* fallback */
-        return PyObject_HasAttrString(obj, "endian");
-    return PyObject_IsInstance(obj, bitarray_basetype);
-}
-
 static void
-setunused(bitarrayobject *self)
+setunused(bitarrayobject *a)
 {
-    const idx_t n = BITS(Py_SIZE(self));
+    const idx_t n = BITS(Py_SIZE(a));
     idx_t i;
 
-    for (i = self->nbits; i < n; i++)
-        setbit(self, i, 0);
+    for (i = a->nbits; i < n; i++)
+        setbit(a, i, 0);
 }
 
 static unsigned char bitcount_lookup[256] = {
-    0, 1, 1, 2, 1, 2, 2, 3, 1, 2, 2, 3, 2, 3, 3, 4,
-    1, 2, 2, 3, 2, 3, 3, 4, 2, 3, 3, 4, 3, 4, 4, 5,
-    1, 2, 2, 3, 2, 3, 3, 4, 2, 3, 3, 4, 3, 4, 4, 5,
-    2, 3, 3, 4, 3, 4, 4, 5, 3, 4, 4, 5, 4, 5, 5, 6,
-    1, 2, 2, 3, 2, 3, 3, 4, 2, 3, 3, 4, 3, 4, 4, 5,
-    2, 3, 3, 4, 3, 4, 4, 5, 3, 4, 4, 5, 4, 5, 5, 6,
-    2, 3, 3, 4, 3, 4, 4, 5, 3, 4, 4, 5, 4, 5, 5, 6,
-    3, 4, 4, 5, 4, 5, 5, 6, 4, 5, 5, 6, 5, 6, 6, 7,
-    1, 2, 2, 3, 2, 3, 3, 4, 2, 3, 3, 4, 3, 4, 4, 5,
-    2, 3, 3, 4, 3, 4, 4, 5, 3, 4, 4, 5, 4, 5, 5, 6,
-    2, 3, 3, 4, 3, 4, 4, 5, 3, 4, 4, 5, 4, 5, 5, 6,
-    3, 4, 4, 5, 4, 5, 5, 6, 4, 5, 5, 6, 5, 6, 6, 7,
-    2, 3, 3, 4, 3, 4, 4, 5, 3, 4, 4, 5, 4, 5, 5, 6,
-    3, 4, 4, 5, 4, 5, 5, 6, 4, 5, 5, 6, 5, 6, 6, 7,
-    3, 4, 4, 5, 4, 5, 5, 6, 4, 5, 5, 6, 5, 6, 6, 7,
-    4, 5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8,
+    0,1,1,2,1,2,2,3,1,2,2,3,2,3,3,4,1,2,2,3,2,3,3,4,2,3,3,4,3,4,4,5,
+    1,2,2,3,2,3,3,4,2,3,3,4,3,4,4,5,2,3,3,4,3,4,4,5,3,4,4,5,4,5,5,6,
+    1,2,2,3,2,3,3,4,2,3,3,4,3,4,4,5,2,3,3,4,3,4,4,5,3,4,4,5,4,5,5,6,
+    2,3,3,4,3,4,4,5,3,4,4,5,4,5,5,6,3,4,4,5,4,5,5,6,4,5,5,6,5,6,6,7,
+    1,2,2,3,2,3,3,4,2,3,3,4,3,4,4,5,2,3,3,4,3,4,4,5,3,4,4,5,4,5,5,6,
+    2,3,3,4,3,4,4,5,3,4,4,5,4,5,5,6,3,4,4,5,4,5,5,6,4,5,5,6,5,6,6,7,
+    2,3,3,4,3,4,4,5,3,4,4,5,4,5,5,6,3,4,4,5,4,5,5,6,4,5,5,6,5,6,6,7,
+    3,4,4,5,4,5,5,6,4,5,5,6,5,6,6,7,4,5,5,6,5,6,6,7,5,6,6,7,6,7,7,8,
 };
 
+/*********** end of code basically copied from _bitarray.c *************/
+
+/* set using the Python module function _set_babt() */
+static PyObject *bitarray_basetype = NULL;
+
+/* Return 1 if obj is a bitarray, 0 otherwise.
+   Note that this is implemented differently in _bitarray.c */
+static int
+bitarray_Check(PyObject *obj)
+{
+    if (bitarray_basetype == NULL) {
+        fprintf(stderr, "FATAL: bitarray_basetype missing\n");
+        exit(1);
+    }
+    return PyObject_IsInstance(obj, bitarray_basetype);
+}
+
+/************ start of actual functionality in this module *************/
+
 /* return the smallest index i for which a.count(1, 0, i) == n, or when
    n exceeds the total count return -1  */
 static idx_t
@@ -162,18 +165,18 @@ count_to_n(bitarrayobject *a, idx_t n)
 
 /* return index of last occurrence of vi, -1 when x is not in found. */
 static idx_t
-find_last(bitarrayobject *self, int vi)
+find_last(bitarrayobject *a, int vi)
 {
     Py_ssize_t j;
     idx_t i;
     char c;
 
-    if (self->nbits == 0)
+    if (a->nbits == 0)
         return -1;
 
     /* search within top byte */
-    for (i = self->nbits - 1; i >= BITS(self->nbits / 8); i--)
-        if (GETBIT(self, i) == vi)
+    for (i = a->nbits - 1; i >= BITS(a->nbits / 8); i--)
+        if (GETBIT(a, i) == vi)
             return i;
 
     if (i < 0)  /* not found within top byte */
@@ -186,7 +189,7 @@ find_last(bitarrayobject *self, int vi)
 
     /* skip ahead by checking whole bytes */
     for (j = BYTES(i) - 1; j >= 0; j--)
-        if (c ^ self->ob_item[j])
+        if (c ^ a->ob_item[j])
             break;
 
     if (j < 0)  /* not found within bytes */
@@ -194,16 +197,30 @@ find_last(bitarrayobject *self, int vi)
 
     /* search within byte found */
     for (i = BITS(j + 1) - 1; i >= BITS(j); i--)
-        if (GETBIT(self, i) == vi)
+        if (GETBIT(a, i) == vi)
             return i;
 
     return -1;
 }
 
+/* translation table which swaps the 4 highest with the 4 lowest bits in
+   each byte - to be used as argument of bytes.translate() */
+static PyObject *
+make_swap_hilo_bytes(void)
+{
+    char bytes[256];
+    int i;
+
+    for (i = 0; i < 256; i++)
+        bytes[i] = (char) (((i & 0x0f) << 4) ^ (i >> 4));
+
+    return PyBytes_FromStringAndSize(bytes, 256);
+}
+
 /*************************** Module functions **********************/
 
 static PyObject *
-count_n(PyObject *self, PyObject *args)
+count_n(PyObject *module, PyObject *args)
 {
     PyObject *a;
     idx_t n, i;
@@ -212,7 +229,7 @@ count_n(PyObject *self, PyObject *args)
         return NULL;
 
     if (!bitarray_Check(a)) {
-        PyErr_SetString(PyExc_TypeError, "bitarray object expected");
+        PyErr_SetString(PyExc_TypeError, "bitarray expected");
         return NULL;
     }
     if (n < 0) {
@@ -241,17 +258,17 @@ Raises `ValueError`, when n exceeds the `a.count()`.");
 
 
 static PyObject *
-r_index(PyObject *self, PyObject *args)
+r_index(PyObject *module, PyObject *args)
 {
     PyObject *a, *x = Py_True;
     idx_t i;
-    long vi;
+    int vi;
 
     if (!PyArg_ParseTuple(args, "O|O:rindex", &a, &x))
         return NULL;
 
     if (!bitarray_Check(a)) {
-        PyErr_SetString(PyExc_TypeError, "bitarray object expected");
+        PyErr_SetString(PyExc_TypeError, "bitarray expected");
         return NULL;
     }
     vi = PyObject_IsTrue(x);
@@ -260,7 +277,7 @@ r_index(PyObject *self, PyObject *args)
 
     i = find_last((bitarrayobject *) a, vi);
     if (i < 0) {
-        PyErr_SetString(PyExc_ValueError, "index(x): x not in bitarray");
+        PyErr_Format(PyExc_ValueError, "%d not in bitarray", vi);
         return NULL;
     }
     return PyLong_FromLongLong(i);
@@ -284,14 +301,14 @@ static PyObject *
 two_bitarray_func(PyObject *args, enum kernel_type kern, char *format)
 {
     PyObject *a, *b;
-    Py_ssize_t i;
+    Py_ssize_t n, i;
     idx_t res = 0;
     unsigned char c;
 
     if (!PyArg_ParseTuple(args, format, &a, &b))
         return NULL;
     if (!(bitarray_Check(a) && bitarray_Check(b))) {
-        PyErr_SetString(PyExc_TypeError, "bitarray object expected");
+        PyErr_SetString(PyExc_TypeError, "bitarray expected");
         return NULL;
     }
 #define aa  ((bitarrayobject *) a)
@@ -308,27 +325,30 @@ two_bitarray_func(PyObject *args, enum kernel_type kern, char *format)
     }
     setunused(aa);
     setunused(bb);
+    assert(Py_SIZE(a) == Py_SIZE(b));
+    n = Py_SIZE(a);
+
     switch (kern) {
     case KERN_cand:
-        for (i = 0; i < Py_SIZE(aa); i++) {
+        for (i = 0; i < n; i++) {
             c = aa->ob_item[i] & bb->ob_item[i];
             res += bitcount_lookup[c];
         }
         break;
     case KERN_cor:
-        for (i = 0; i < Py_SIZE(aa); i++) {
+        for (i = 0; i < n; i++) {
             c = aa->ob_item[i] | bb->ob_item[i];
             res += bitcount_lookup[c];
         }
         break;
     case KERN_cxor:
-        for (i = 0; i < Py_SIZE(aa); i++) {
+        for (i = 0; i < n; i++) {
             c = aa->ob_item[i] ^ bb->ob_item[i];
             res += bitcount_lookup[c];
         }
         break;
     case KERN_subset:
-        for (i = 0; i < Py_SIZE(aa); i++)
+        for (i = 0; i < n; i++)
             if ((aa->ob_item[i] & bb->ob_item[i]) != aa->ob_item[i])
                 Py_RETURN_FALSE;
         Py_RETURN_TRUE;
@@ -342,7 +362,7 @@ two_bitarray_func(PyObject *args, enum kernel_type kern, char *format)
 
 #define COUNT_FUNC(oper, ochar)                                         \
 static PyObject *                                                       \
-count_ ## oper (bitarrayobject *self, PyObject *args)                   \
+count_ ## oper (bitarrayobject *module, PyObject *args)                 \
 {                                                                       \
     return two_bitarray_func(args, KERN_c ## oper, "OO:count_" #oper);  \
 }                                                                       \
@@ -358,7 +378,7 @@ COUNT_FUNC(xor, "^");
 
 
 static PyObject *
-subset(PyObject *self, PyObject *args)
+subset(PyObject *module, PyObject *args)
 {
     return two_bitarray_func(args, KERN_subset, "OO:subset");
 }
@@ -374,7 +394,7 @@ intermediate bitarray object gets created.");
 
 /* set bitarray_basetype (babt) */
 static PyObject *
-set_babt(PyObject *self, PyObject *obj)
+set_babt(PyObject *module, PyObject *obj)
 {
     bitarray_basetype = obj;
     Py_RETURN_NONE;
@@ -397,10 +417,12 @@ static PyMethodDef module_functions[] = {
 static PyModuleDef moduledef = {
     PyModuleDef_HEAD_INIT, "_util", 0, -1, module_functions,
 };
+#endif
+
 PyMODINIT_FUNC
+#ifdef IS_PY3K
 PyInit__util(void)
 #else
-PyMODINIT_FUNC
 init_util(void)
 #endif
 {
@@ -416,6 +438,7 @@ init_util(void)
         return;
 #endif
 
+    PyModule_AddObject(m, "_swap_hilo_bytes", make_swap_hilo_bytes());
 #ifdef IS_PY3K
     return m;
 #endif
