@@ -27,6 +27,7 @@ if is_py3k:
     unicode = str
 else:
     from cStringIO import StringIO
+    range = xrange
 
 
 from bitarray import (bitarray, frozenbitarray, bitdiff, bits2bytes,
@@ -98,7 +99,11 @@ class Util(object):
 
 class TestsModuleFunctions(unittest.TestCase, Util):
 
-    def test_default_endian(self):
+    def test_version_string(self):
+        # the version string is not a function, but test it here anyway
+        self.assertIsInstance(__version__, str)
+
+    def test_set_default_endian(self):
         self.assertRaises(TypeError, _set_default_endian, 0)
         self.assertRaises(TypeError, _set_default_endian, 'little', 0)
         self.assertRaises(ValueError, _set_default_endian, 'foo')
@@ -106,19 +111,27 @@ class TestsModuleFunctions(unittest.TestCase, Util):
             _set_default_endian(default_endian)
             a = bitarray()
             self.assertEqual(a.endian(), default_endian)
-            a = bitarray('10111')
-            self.assertEqual(a.endian(), default_endian)
+            for x in None, 0, 64, '10111', [1, 0]:
+                a = bitarray(x)
+                self.assertEqual(a.endian(), default_endian)
 
             for endian in 'big', 'little':
                 a = bitarray(endian=endian)
                 self.assertEqual(a.endian(), endian)
 
-            # test get_default_endian()
-            self.assertEqual(get_default_endian(), default_endian)
             # make sure that calling _set_default_endian wrong does not
             # change the default endianness
             self.assertRaises(ValueError, _set_default_endian, 'foobar')
-            self.assertEqual(get_default_endian(), default_endian)
+            self.assertEqual(bitarray().endian(), default_endian)
+
+    def test_get_default_endian(self):
+        # takes no arguments
+        self.assertRaises(TypeError, get_default_endian, 'big')
+        for default_endian in 'big', 'little':
+            _set_default_endian(default_endian)
+            endian = get_default_endian()
+            self.assertEqual(endian, default_endian)
+            self.assertIsInstance(endian, str)
 
     def test_bitdiff(self):
         a = bitarray('0011')
@@ -192,11 +205,27 @@ class CreateObjectTests(unittest.TestCase, Util):
         self.assertNotEqual(a, b)
         self.assertEqual(a.tobytes(), b.tobytes())
 
+    def test_endian_default(self):
+        _set_default_endian('big')
+        a_big = bitarray()
+        _set_default_endian('little')
+        a_little = bitarray()
+        _set_default_endian('big')
+
+        self.assertEqual(a_big.endian(), 'big')
+        self.assertEqual(a_little.endian(), 'little')
+
     def test_endian_wrong(self):
         self.assertRaises(TypeError, bitarray.__new__, bitarray, endian=0)
         self.assertRaises(ValueError, bitarray.__new__, bitarray, endian='')
-        self.assertRaises(ValueError, bitarray.__new__,
-                          bitarray, endian='foo')
+        self.assertRaises(ValueError,
+                          bitarray.__new__, bitarray, endian='foo')
+        try:
+            bitarray(Ellipsis)
+        except TypeError as e:
+            self.assertTrue('ellipsis' in str(e))
+        else:
+            raise SystemError("we shouldn'e get here")
 
     def test_integers(self):
         for n in range(50):
@@ -258,6 +287,10 @@ class CreateObjectTests(unittest.TestCase, Util):
         a = bitarray(itertools.repeat('0', 10))
         self.assertEqual(a, bitarray(10 * '1'))
 
+    def test_range(self):
+        a = bitarray(range(-3, 3))
+        self.assertEqual(a, bitarray('111011'))
+
     def test_01(self):
         a = bitarray('0010111')
         self.assertEqual(a.tolist(), [0, 0, 1, 0, 1, 1, 1])
@@ -376,7 +409,7 @@ tests.append(ToObjectsTests)
 class MetaDataTests(unittest.TestCase, Util):
 
     def test_buffer_info1(self):
-        a = bitarray('0000111100001', endian='little')
+        a = bitarray(13, endian='little')
         self.assertEqual(a.buffer_info()[1:4], (2, 'little', 3))
 
         a = bitarray()
@@ -385,38 +418,25 @@ class MetaDataTests(unittest.TestCase, Util):
         bi = a.buffer_info()
         self.assertIsInstance(bi, tuple)
         self.assertEqual(len(bi), 5)
-
         self.assertIsInstance(bi[0], int)
-        if is_py3k:
-            self.assertIsInstance(bi[1], int)
-        self.assertIsInstance(bi[2], str)
-        self.assertIsInstance(bi[3], int)
-        if is_py3k:
-            self.assertIsInstance(bi[4], int)
 
     def test_buffer_info2(self):
-        for n in range(50):
-            bi = bitarray(n).buffer_info()
-            self.assertEqual(bi[1], bits2bytes(n))
-            self.assertEqual(bi[3] + n, 8 * bi[1])
-            self.assertTrue(bi[4] >= bi[1])
-
-    def test_buffer_info3(self):
-        a = bitarray(endian='little')
-        self.assertEqual(a.buffer_info()[2], 'little')
-
-        a = bitarray(endian='big')
-        self.assertEqual(a.buffer_info()[2], 'big')
-
+        for endian in 'big', 'little':
+            for n in range(50):
+                bi = bitarray(n, endian).buffer_info()
+                self.assertEqual(bi[1], bits2bytes(n))  # bytes
+                self.assertEqual(bi[2], endian)         # endianness
+                self.assertEqual(bi[3], 8 * bi[1] - n)  # unused
+                self.assertTrue(bi[4] >= bi[1])         # allocated
 
     def test_endian(self):
-        a = bitarray(endian='little')
-        self.assertEqual(a.endian(), 'little')
+        for endian in 'big', 'little':
+            a = bitarray(endian=endian)
+            self.assertEqual(a.endian(), endian)
 
         a = bitarray(endian='big')
         self.assertEqual(a.endian(), 'big')
 
-
     def test_length(self):
         for n in range(100):
             a = bitarray(n)
@@ -1448,7 +1468,7 @@ class MethodTests(unittest.TestCase, Util):
                 self.assertEqual(res1, res2)
 
 
-    def test_count1(self):
+    def test_count_basic(self):
         a = bitarray('10011')
         self.assertEqual(a.count(), 3)
         self.assertEqual(a.count(True), 3)
@@ -1461,19 +1481,29 @@ class MethodTests(unittest.TestCase, Util):
         self.assertRaises(TypeError, a.count, 0, 'A')
         self.assertRaises(TypeError, a.count, 0, 0, 'A')
 
-    def test_count2(self):
-        for i in range(0, 256):
+    def test_count_byte(self):
+
+        def count(n):  # count 1 bits in number
+            cnt = 0
+            while n:
+                cnt += n & 1
+                n >>= 1
+            return cnt
+
+        for i in range(256):
             a = bitarray()
             a.frombytes(bytes(bytearray([i])))
-            self.assertEqual(a.count(), a.to01().count('1'))
+            self.assertEqual(len(a), 8)
+            self.assertEqual(a.count(), count(i))
+            self.assertEqual(a.count(), bin(i)[2:].count('1'))
 
-    def test_count3(self):
+    def test_count_whole_range(self):
         for a in self.randombitarrays():
             s = a.to01()
             self.assertEqual(a.count(1), s.count('1'))
             self.assertEqual(a.count(0), s.count('0'))
 
-    def test_count4(self):
+    def test_count_allones(self):
         N = 37
         a = bitarray(N)
         a.setall(1)
@@ -1481,7 +1511,7 @@ class MethodTests(unittest.TestCase, Util):
             for j in range(i, N):
                 self.assertEqual(a.count(1, i, j), j - i)
 
-    def test_count5(self):
+    def test_count_explicit(self):
         for endian in 'big', 'little':
             a = bitarray('01001100' '01110011' '01', endian)
             self.assertEqual(a.count(), 9)
@@ -1493,7 +1523,7 @@ class MethodTests(unittest.TestCase, Util):
             self.assertEqual(a.count(1, 1, -1), 8)
             self.assertEqual(a.count(1, 17, 14), 0)
 
-    def test_count6(self):
+    def test_count_random(self):
         for a in self.randombitarrays():
             s = a.to01()
             i = randint(-3, len(a) + 1)
@@ -1731,6 +1761,17 @@ class MethodTests(unittest.TestCase, Util):
             self.check_obj(a)
 
 
+    def test_clear(self):
+        for a in self.randombitarrays():
+            ida = id(a)
+            endian = a.endian()
+            a.clear()
+            self.assertEqual(a, bitarray())
+            self.assertEqual(id(a), ida)
+            self.assertEqual(a.endian(), endian)
+            self.assertEqual(len(a), 0)
+
+
     def test_setall(self):
         a = bitarray(5)
         a.setall(True)
@@ -2084,17 +2125,18 @@ class PrefixCodeTests(unittest.TestCase, Util):
 
     def test_encode_iter(self):
         a = bitarray()
-        d = {'a': bitarray('0'), 'b': bitarray('1')}
-        a.encode(d, iter('abba'))
+        d = {0: bitarray('0'), 1: bitarray('1')}
+        a.encode(d, iter([0, 1, 1, 0]))
         self.assertEqual(a, bitarray('0110'))
 
         def foo():
-            for c in 'bbaabb':
+            for c in 1, 1, 0, 0, 1, 1:
                 yield c
 
         a.encode(d, foo())
-        self.assertEqual(a, bitarray('0110110011'))
-        self.assertEqual(d, {'a': bitarray('0'), 'b': bitarray('1')})
+        a.encode(d, range(2))
+        self.assertEqual(a, bitarray('011011001101'))
+        self.assertEqual(d, {0: bitarray('0'), 1: bitarray('1')})
 
     def test_encode(self):
         d = {'I': bitarray('1'),
@@ -2326,6 +2368,13 @@ class BufferInterfaceTests(unittest.TestCase):
         self.assertEqual(v[34:67].tobytes(), b.tobytes())
         self.assertEqual(v.tobytes(), a.tobytes())
 
+    def test_resize(self):
+        a = bitarray('01000001' '01000010' '01000011', endian='big')
+        v = memoryview(a)
+        self.assertRaises(BufferError, a.append, 1)
+        self.assertRaises(BufferError, a.clear)
+        self.assertRaises(BufferError, a.__delitem__, slice(0, 8))
+
     def test_write(self):
         a = bitarray(8000)
         a.setall(0)
@@ -2391,8 +2440,9 @@ class TestsFrozenbitarray(unittest.TestCase, Util):
     def test_immutable(self):
         a = frozenbitarray('111')
         self.assertRaises(TypeError, a.append, True)
-        self.assertRaises(TypeError, a.__setitem__, 0, 0)
+        self.assertRaises(TypeError, a.clear)
         self.assertRaises(TypeError, a.__delitem__, 0)
+        self.assertRaises(TypeError, a.__setitem__, 0, 0)
 
     def test_dictkey(self):
         a = frozenbitarray('01')
