@@ -279,6 +279,25 @@ tests.append(TestsHelpers)
 
 class TestsBitwiseCount(unittest.TestCase, Util):
 
+    def test_count_byte(self):
+        ones = bitarray(8)
+        ones.setall(1)
+        zeros = bitarray(8)
+        zeros.setall(0)
+        for i in range(0, 256):
+            a = bitarray()
+            a.frombytes(bytes(bytearray([i])))
+            cnt = a.count()
+            self.assertEqual(count_and(a, zeros), 0)
+            self.assertEqual(count_and(a, ones), cnt)
+            self.assertEqual(count_and(a, a), cnt)
+            self.assertEqual(count_or(a, zeros), cnt)
+            self.assertEqual(count_or(a, ones), 8)
+            self.assertEqual(count_or(a, a), cnt)
+            self.assertEqual(count_xor(a, zeros), cnt)
+            self.assertEqual(count_xor(a, ones), 8 - cnt)
+            self.assertEqual(count_xor(a, a), 0)
+
     def test_bit_count1(self):
         a = bitarray('001111')
         aa = a.copy()
@@ -379,6 +398,22 @@ tests.append(TestsSubset)
 
 class TestsHexlify(unittest.TestCase, Util):
 
+    def test_swap_hilo_bytes(self):
+        from bitarray._util import _swap_hilo_bytes
+
+        self.assertEqual(len(_swap_hilo_bytes), 256)
+        for i in range(256):
+            byte = bytes(bytearray([i]))
+            a = bitarray()
+            a.frombytes(byte)
+            self.assertEqual(len(a), 8)
+
+            b = a[4:8] + a[0:4]
+            self.assertEqual(b.tobytes(),
+                             byte.translate(_swap_hilo_bytes))
+            # with just _swap_hilo_bytes[i] we'd get an integer on Py3
+            self.assertEqual(b.tobytes(), _swap_hilo_bytes[i:i + 1])
+
     def test_ba2hex(self):
         self.assertEqual(ba2hex(bitarray(0, 'big')), '')
         self.assertEqual(ba2hex(bitarray('1110', 'big')), 'e')
